@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Designer of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qdesigner_utils_p.h"
 #include "qdesigner_propertycommand_p.h"
@@ -48,6 +23,7 @@
 
 #include <QtWidgets/qapplication.h>
 #include <QtGui/qicon.h>
+#include <QtGui/qpalette.h>
 #include <QtGui/qpixmap.h>
 #include <QtWidgets/qlistwidget.h>
 #include <QtWidgets/qtreewidget.h>
@@ -56,19 +32,21 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 namespace qdesigner_internal
 {
     // ### FIXME Qt 8: Remove (QTBUG-96005)
     QString legacyDataDirectory()
     {
-        return QDir::homePath() + u"/.designer"_qs;
+        return QDir::homePath() + u"/.designer"_s;
     }
 
     QString dataDirectory()
     {
 #if QT_VERSION >= QT_VERSION_CHECK(7, 0, 0)
         return QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation)
-               + u'/' + QCoreApplication::organizationName() + u"/Designer"_qs;
+               + u'/' + QCoreApplication::organizationName() + u"/Designer"_s;
 #else
         return legacyDataDirectory();
 #endif
@@ -800,6 +778,22 @@ namespace qdesigner_internal
             m_widget->setUpdatesEnabled(true);
     }
 
+// from qpalette.cpp
+quint64 paletteResolveMask(QPalette::ColorGroup colorGroup,
+                           QPalette::ColorRole colorRole)
+{
+    const auto offset = quint64(QPalette::NColorRoles) * quint64(colorGroup);
+    const auto bitPos = quint64(colorRole) + offset;
+    return 1ull << bitPos;
+}
+
+quint64 paletteResolveMask(QPalette::ColorRole colorRole)
+{
+    return paletteResolveMask(QPalette::Active, colorRole)
+        | paletteResolveMask(QPalette::Inactive, colorRole)
+        | paletteResolveMask(QPalette::Disabled, colorRole);
+}
+
 } // namespace qdesigner_internal
 
 QT_END_NAMESPACE
