@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the tools applications of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "clangcodeparser.h"
 
@@ -57,6 +21,8 @@
 #include <QtCore/qfile.h>
 #include <QtCore/qscopedvaluerollback.h>
 #include <QtCore/qtemporarydir.h>
+#include <QtCore/qtextstream.h>
+#include <QtCore/qvarlengtharray.h>
 
 #include <clang-c/Index.h>
 
@@ -72,7 +38,7 @@ static CXTranslationUnit_Flags flags_ = static_cast<CXTranslationUnit_Flags>(0);
 static CXIndex index_ = nullptr;
 
 QByteArray ClangCodeParser::s_fn;
-constexpr const char *fnDummyFileName = "/fn_dummyfile.cpp";
+constexpr const char fnDummyFileName[] = "/fn_dummyfile.cpp";
 
 #ifndef QT_NO_DEBUG_STREAM
 template<class T>
@@ -200,6 +166,48 @@ static Access fromCX_CXXAccessSpecifier(CX_CXXAccessSpecifier spec)
 /*!
    Returns the spelling in the file for a source range
  */
+
+struct FileCacheEntry
+{
+    QByteArray fileName;
+    QByteArray content;
+};
+
+static inline QString fromCache(const QByteArray &cache,
+                                unsigned int offset1, unsigned int offset2)
+{
+    return QString::fromUtf8(cache.mid(offset1, offset2 - offset1));
+}
+
+static QString readFile(CXFile cxFile, unsigned int offset1, unsigned int offset2)
+{
+    using FileCache = QList<FileCacheEntry>;
+    static FileCache cache;
+
+    CXString cxFileName = clang_getFileName(cxFile);
+    const QByteArray fileName = clang_getCString(cxFileName);
+    clang_disposeString(cxFileName);
+
+    for (const auto &entry : std::as_const(cache)) {
+        if (fileName == entry.fileName)
+            return fromCache(entry.content, offset1, offset2);
+    }
+
+    // "fn_dummyfile.cpp" comes with varying cxFile values
+    if (fileName == fnDummyFileName)
+        return fromCache(ClangCodeParser::fn(), offset1, offset2);
+
+    QFile file(QString::fromUtf8(fileName));
+    if (file.open(QIODeviceBase::ReadOnly)) { // binary to match clang offsets
+        FileCacheEntry entry{fileName, file.readAll()};
+        cache.prepend(entry);
+        while (cache.size() > 5)
+            cache.removeLast();
+        return fromCache(entry.content, offset1, offset2);
+    }
+    return {};
+}
+
 static QString getSpelling(CXSourceRange range)
 {
     auto start = clang_getRangeStart(range);
@@ -211,14 +219,8 @@ static QString getSpelling(CXSourceRange range)
 
     if (file1 != file2 || offset2 <= offset1)
         return QString();
-    QFile file(fromCXString(clang_getFileName(file1)));
-    if (!file.open(QFile::ReadOnly)) {
-        if (file.fileName() == fnDummyFileName)
-            return QString::fromUtf8(ClangCodeParser::fn().mid(offset1, offset2 - offset1));
-        return QString();
-    }
-    file.seek(offset1);
-    return QString::fromUtf8(file.read(offset2 - offset1));
+
+    return readFile(file1, offset1, offset2);
 }
 
 /*!
@@ -324,7 +326,7 @@ static Node *findNodeForCursor(QDocDatabase *qdb, CXCursor cur)
         auto numArg = clang_getNumArgTypes(funcType);
         bool isVariadic = clang_isFunctionTypeVariadic(funcType);
         QVarLengthArray<QString, 20> args;
-        for (Node *candidate : qAsConst(candidates)) {
+        for (Node *candidate : std::as_const(candidates)) {
             if (!candidate->isFunction(Node::CPP))
                 continue;
             auto fn = static_cast<FunctionNode *>(candidate);
@@ -335,6 +337,11 @@ static Node *findNodeForCursor(QDocDatabase *qdb, CXCursor cur)
                 continue;
             if (isVariadic && parameters.last().type() != QLatin1String("..."))
                 continue;
+            if (fn->isRef() != (clang_Type_getCXXRefQualifier(funcType) == CXRefQualifier_LValue))
+                continue;
+            if (fn->isRefRef() != (clang_Type_getCXXRefQualifier(funcType) == CXRefQualifier_RValue))
+                continue;
+
             bool different = false;
             for (int i = 0; i < numArg; ++i) {
                 CXType argType = clang_getArgType(funcType, i);
@@ -746,7 +753,7 @@ CXChildVisitResult ClangVisitor::visitHeader(CXCursor cursor, CXSourceLocation l
 #endif
     case CXCursor_EnumDecl: {
         auto *en = static_cast<EnumNode *>(findNodeForCursor(qdb_, cursor));
-        if (en && en->items().count())
+        if (en && en->items().size())
             return CXChildVisit_Continue; // Was already parsed, probably in another TU
         QString enumTypeName = fromCXString(clang_getCursorSpelling(cursor));
         if (enumTypeName.isEmpty()) {
@@ -969,32 +976,44 @@ bool ClangVisitor::parseProperty(const QString &spelling, const Location &loc)
 
     QString signature = spelling.mid(lpIdx + 1, rpIdx - lpIdx - 1);
     signature = signature.simplified();
-
-    QString type;
-    QString name;
     QStringList parts = signature.split(QChar(' '), Qt::SkipEmptyParts);
-    if (parts.size() < 2)
-        return false;
-    if (parts.first() == QLatin1String("enum"))
-        parts.removeFirst(); // QTBUG-80027
 
-    type = parts.takeFirst();
-    if (type == QLatin1String("const") && !parts.empty())
-        type += " " + parts.takeFirst();
+    static const QStringList attrs =
+            QStringList() << "READ"     << "MEMBER"     << "WRITE"
+                          << "NOTIFY"   << "CONSTANT"   << "FINAL"
+                          << "REQUIRED" << "BINDABLE"   << "DESIGNABLE"
+                          << "RESET"    << "REVISION"   << "SCRIPTABLE"
+                          << "STORED"   << "USER";
 
-    if (!parts.empty())
-        name = parts.takeFirst();
-    else
+    // Find the location of the first attribute. All preceding parts
+    // represent the property type + name.
+    auto it = std::find_if(parts.cbegin(), parts.cend(),
+                           [](const QString &attr) -> bool {
+        return attrs.contains(attr);
+    });
+
+    if (it == parts.cend() || std::distance(parts.cbegin(), it) < 2)
         return false;
 
-    if (name.front() == QChar('*')) {
-        type.append(QChar('*'));
+    QStringList typeParts;
+    std::copy(parts.cbegin(), it, std::back_inserter(typeParts));
+    parts.erase(parts.cbegin(), it);
+    QString name = typeParts.takeLast();
+
+    // Move the pointer operator(s) from name to type
+    while (!name.isEmpty() && name.front() == QChar('*')) {
+        typeParts.last().push_back(name.front());
         name.remove(0, 1);
     }
+
+    // Need at least READ or MEMBER + getter/member name
+    if (parts.size() < 2 || name.isEmpty())
+        return false;
+
     auto *property = new PropertyNode(parent_, name);
     property->setAccess(Access::Public);
     property->setLocation(loc);
-    property->setDataType(type);
+    property->setDataType(typeParts.join(QChar(' ')));
 
     int i = 0;
     while (i < parts.size()) {
@@ -1012,6 +1031,8 @@ bool ClangVisitor::parseProperty(const QString &spelling, const Location &loc)
             } else if (key == "WRITE") {
                 qdb_->addPropertyFunction(property, value, PropertyNode::Setter);
                 property->setWritable(true);
+            } else if (key == "MEMBER") {
+                property->setWritable(true);
             } else if (key == "STORED") {
                 property->setStored(value.toLower() == "true");
             } else if (key == "DESIGNABLE") {
@@ -1089,8 +1110,8 @@ Node *ClangVisitor::nodeForCommentAtLocation(CXSourceLocation loc, CXSourceLocat
   \a config. Call the initializeParser() in the base class.
   Get the defines list from the qdocconf database.
 
-  \note on \macos, we try to also query the system/framework
-  include paths from the compiler.
+  \note on \macos and Linux, we try to also query the system
+  and framework (\macos) include paths from the compiler.
  */
 void ClangCodeParser::initializeParser()
 {
@@ -1100,9 +1121,11 @@ void ClangCodeParser::initializeParser()
                                             Config::IncludePaths);
 #ifdef Q_OS_MACOS
     args.append(Utilities::getInternalIncludePaths(QStringLiteral("clang++")));
+#elif defined(Q_OS_LINUX)
+    args.append(Utilities::getInternalIncludePaths(QStringLiteral("g++")));
 #endif
     m_includePaths.clear();
-    for (const auto &path : qAsConst(args)) {
+    for (const auto &path : std::as_const(args)) {
         if (!path.isEmpty())
             m_includePaths.append(path.toUtf8());
     }
@@ -1198,7 +1221,17 @@ void ClangCodeParser::parseHeaderFile(const Location & /*location*/, const QStri
 }
 
 static const char *defaultArgs_[] = {
+/*
+  https://bugreports.qt.io/browse/QTBUG-94365
+  An unidentified bug in Clang 15.x causes parsing failures due to errors in
+  the AST. This replicates only with C++20 support enabled - avoid the issue
+  by using C++17 with Clang 15.
+ */
+#if LIBCLANG_VERSION_MAJOR == 15
+    "-std=c++17",
+#else
     "-std=c++20",
+#endif
 #ifndef Q_OS_WIN
     "-fPIC",
 #else
@@ -1228,7 +1261,7 @@ void ClangCodeParser::getDefaultArgs()
     m_args.clear();
     m_args.insert(m_args.begin(), std::begin(defaultArgs_), std::end(defaultArgs_));
     // Add the defines from the qdocconf file.
-    for (const auto &p : qAsConst(m_defines))
+    for (const auto &p : std::as_const(m_defines))
         m_args.push_back(p.constData());
 }
 
@@ -1344,8 +1377,7 @@ void ClangCodeParser::buildPCH()
             m_args.push_back("-xc++");
             CXTranslationUnit tu;
             QString tmpHeader = m_pchFileDir->path() + "/" + module;
-            QFile tmpHeaderFile(tmpHeader);
-            if (tmpHeaderFile.open(QIODevice::Text | QIODevice::WriteOnly)) {
+            if (QFile tmpHeaderFile(tmpHeader); tmpHeaderFile.open(QIODevice::Text | QIODevice::WriteOnly)) {
                 QTextStream out(&tmpHeaderFile);
                 if (header.isEmpty()) {
                     for (auto it = m_allHeaders.constKeyValueBegin();
@@ -1365,7 +1397,6 @@ void ClangCodeParser::buildPCH()
                     }
                     out << QLatin1String("#include \"") + header + QLatin1String("\"");
                 }
-                tmpHeaderFile.close();
             }
 
             CXErrorCode err =
@@ -1409,7 +1440,7 @@ void ClangCodeParser::precompileHeaders()
 {
     getDefaultArgs();
     getMoreArgs();
-    for (const auto &p : qAsConst(m_moreArgs))
+    for (const auto &p : std::as_const(m_moreArgs))
         m_args.push_back(p.constData());
 
     flags_ = static_cast<CXTranslationUnit_Flags>(CXTranslationUnit_Incomplete
@@ -1457,7 +1488,7 @@ void ClangCodeParser::parseSourceFile(const Location & /*location*/, const QStri
         m_args.push_back(m_pchName.constData());
     }
     getMoreArgs();
-    for (const auto &p : qAsConst(m_moreArgs))
+    for (const auto &p : std::as_const(m_moreArgs))
         m_args.push_back(p.constData());
 
     CXTranslationUnit tu;
@@ -1525,7 +1556,7 @@ void ClangCodeParser::parseSourceFile(const Location & /*location*/, const QStri
             } else if (CodeParser::isWorthWarningAbout(doc)) {
                 bool future = false;
                 if (doc.metaCommandsUsed().contains(COMMAND_SINCE)) {
-                    QString sinceVersion = doc.metaCommandArgs(COMMAND_SINCE)[0].first;
+                    QString sinceVersion = doc.metaCommandArgs(COMMAND_SINCE).at(0).first;
                     if (getUnpatchedVersion(sinceVersion) > getUnpatchedVersion(m_version))
                         future = true;
                 }
@@ -1624,7 +1655,7 @@ Node *ClangCodeParser::parseFnArg(const Location &location, const QString &fnSig
 
     std::vector<const char *> args(std::begin(defaultArgs_), std::end(defaultArgs_));
     // Add the defines from the qdocconf file.
-    for (const auto &p : qAsConst(m_defines))
+    for (const auto &p : std::as_const(m_defines))
         args.push_back(p.constData());
     if (!m_pchName.isEmpty()) {
         args.push_back("-w");
@@ -1633,7 +1664,7 @@ Node *ClangCodeParser::parseFnArg(const Location &location, const QString &fnSig
     }
     CXTranslationUnit tu;
     s_fn.clear();
-    for (const auto &ns : qAsConst(m_namespaceScope))
+    for (const auto &ns : std::as_const(m_namespaceScope))
         s_fn.prepend("namespace " + ns.toUtf8() + " {");
     s_fn += fnSignature.toUtf8();
     if (!s_fn.endsWith(";"))
