@@ -1,64 +1,24 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the tools applications of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
-
-/*
-  cppcodemarker.cpp
-*/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "cppcodemarker.h"
 
-#include "generator.h"
+#include "access.h"
+#include "enumnode.h"
+#include "functionnode.h"
+#include "namespacenode.h"
+#include "propertynode.h"
+#include "qmlpropertynode.h"
 #include "text.h"
 #include "tree.h"
+#include "typedefnode.h"
+#include "variablenode.h"
 
 #include <QtCore/qdebug.h>
-#include <QtCore/qregexp.h>
-
-#include <ctype.h>
+#include <QtCore/qregularexpression.h>
 
 QT_BEGIN_NAMESPACE
 
-/*!
-  The constructor does nothing.
- */
-CppCodeMarker::CppCodeMarker()
-{
-    // nothing.
-}
-
-/*!
-  The destructor does nothing.
- */
-CppCodeMarker::~CppCodeMarker()
-{
-    // nothing.
-}
-
 /*!
   Returns \c true.
  */
@@ -106,12 +66,9 @@ QString CppCodeMarker::markedUpSynopsis(const Node *node, const Node * /* relati
 {
     const int MaxEnumValues = 6;
     const FunctionNode *func;
-    const PropertyNode *property;
     const VariableNode *variable;
     const EnumNode *enume;
-    const TypedefNode *typedeff;
     QString synopsis;
-    QString extra;
     QString name;
 
     name = taggedNode(node);
@@ -121,23 +78,23 @@ QString CppCodeMarker::markedUpSynopsis(const Node *node, const Node * /* relati
 
     if (style == Section::Details) {
         if (!node->isRelatedNonmember() && !node->isProxyNode() && !node->parent()->name().isEmpty()
-            && !node->parent()->isHeader() && !node->isProperty() && !node->isQmlNode()
-            && !node->isJsNode()) {
+            && !node->parent()->isHeader() && !node->isProperty() && !node->isQmlNode()) {
             name.prepend(taggedNode(node->parent()) + "::");
         }
     }
 
     switch (node->nodeType()) {
     case Node::Namespace:
-        synopsis = "namespace " + name;
-        break;
     case Node::Class:
-        synopsis = "class " + name;
+    case Node::Struct:
+    case Node::Union:
+        synopsis = Node::nodeTypeString(node->nodeType());
+        synopsis += QLatin1Char(' ') + name;
         break;
     case Node::Function:
         func = (const FunctionNode *)node;
         if (style == Section::Details) {
-            QString templateDecl = node->templateDecl();
+            const QString &templateDecl = node->templateDecl();
             if (!templateDecl.isEmpty())
                 synopsis = templateDecl + QLatin1Char(' ');
         }
@@ -154,15 +111,10 @@ QString CppCodeMarker::markedUpSynopsis(const Node *node, const Node * /* relati
                     QString name = parameters.at(i).name();
                     QString type = parameters.at(i).type();
                     QString value = parameters.at(i).defaultValue();
-                    QString paramName;
-                    if (!name.isEmpty()) {
-                        synopsis += typified(type, true);
-                        paramName = name;
-                    } else {
-                        paramName = type;
-                    }
-                    if (style != Section::AllMembers || name.isEmpty())
-                        synopsis += "<@param>" + protect(paramName) + "</@param>";
+                    bool trailingSpace = style != Section::AllMembers && !name.isEmpty();
+                    synopsis += typified(type, trailingSpace);
+                    if (style != Section::AllMembers && !name.isEmpty())
+                        synopsis += "<@param>" + protect(name) + "</@param>";
                     if (style != Section::AllMembers && !value.isEmpty())
                         synopsis += " = " + protect(value);
                 }
@@ -193,30 +145,6 @@ QString CppCodeMarker::markedUpSynopsis(const Node *node, const Node * /* relati
                 synopsis.append(" &");
             else if (func->isRefRef())
                 synopsis.append(" &&");
-            QStringList bracketed;
-            if (func->isStatic()) {
-                bracketed += "static";
-            } else if (!func->isNonvirtual()) {
-                if (func->isFinal())
-                    bracketed += "final";
-                if (func->isOverride())
-                    bracketed += "override";
-                if (func->isPureVirtual())
-                    bracketed += "pure";
-                bracketed += "virtual";
-            }
-
-            if (func->access() == Node::Protected)
-                bracketed += "protected";
-            else if (func->access() == Node::Private)
-                bracketed += "private";
-
-            if (func->isSignal())
-                bracketed += "signal";
-            else if (func->isSlot())
-                bracketed += "slot";
-            if (!bracketed.isEmpty())
-                extra += QLatin1Char('[') + bracketed.join(' ') + QStringLiteral("] ");
         }
         break;
     case Node::Enum:
@@ -253,24 +181,28 @@ QString CppCodeMarker::markedUpSynopsis(const Node *node, const Node * /* relati
         }
         break;
     case Node::TypeAlias:
-        if (style == Section::Summary)
-            synopsis = "(alias) ";
-        else if (style == Section::Details)
-            extra = QStringLiteral("[alias] ");
+        if (style == Section::Details) {
+            const QString &templateDecl = node->templateDecl();
+            if (!templateDecl.isEmpty())
+                synopsis += templateDecl + QLatin1Char(' ');
+        }
         synopsis += name;
         break;
     case Node::Typedef:
-        typedeff = static_cast<const TypedefNode *>(node);
-        if (typedeff->associatedEnum()) {
-            synopsis = "flags " + name;
-        } else {
-            synopsis = "typedef " + name;
-        }
+        if (static_cast<const TypedefNode *>(node)->associatedEnum())
+            synopsis = "flags ";
+        synopsis += name;
         break;
-    case Node::Property:
-        property = static_cast<const PropertyNode *>(node);
+    case Node::Property: {
+        auto property = static_cast<const PropertyNode *>(node);
         synopsis = name + " : " + typified(property->qualifiedDataType());
         break;
+    }
+    case Node::QmlProperty: {
+        auto property = static_cast<const QmlPropertyNode *>(node);
+        synopsis = name + " : " + typified(property->dataType());
+        break;
+    }
     case Node::Variable:
         variable = static_cast<const VariableNode *>(node);
         if (style == Section::AllMembers) {
@@ -283,19 +215,12 @@ QString CppCodeMarker::markedUpSynopsis(const Node *node, const Node * /* relati
         synopsis = name;
     }
 
-    if (style == Section::Summary) {
-        if (node->isPreliminary())
-            extra += "(preliminary) ";
-        else if (node->isDeprecated())
-            extra += "(deprecated) ";
-        else if (node->isObsolete())
-            extra += "(obsolete) ";
-    }
-
+    QString extra = CodeMarker::extraSynopsis(node, style);
     if (!extra.isEmpty()) {
         extra.prepend("<@extra>");
         extra.append("</@extra>");
     }
+
     return extra + synopsis;
 }
 
@@ -306,18 +231,18 @@ QString CppCodeMarker::markedUpQmlItem(const Node *node, bool summary)
     QString name = taggedQmlNode(node);
     if (summary) {
         name = linkTag(node, name);
-    } else if (node->isQmlProperty() || node->isJsProperty()) {
-        const QmlPropertyNode *pn = static_cast<const QmlPropertyNode *>(node);
+    } else if (node->isQmlProperty()) {
+        const auto *pn = static_cast<const QmlPropertyNode *>(node);
         if (pn->isAttached())
             name.prepend(pn->element() + QLatin1Char('.'));
     }
     name = "<@name>" + name + "</@name>";
     QString synopsis;
-    if (node->isQmlProperty() || node->isJsProperty()) {
-        const QmlPropertyNode *pn = static_cast<const QmlPropertyNode *>(node);
+    if (node->isQmlProperty()) {
+        const auto *pn = static_cast<const QmlPropertyNode *>(node);
         synopsis = name + " : " + typified(pn->dataType());
-    } else if (node->isFunction(Node::QML) || node->isFunction(Node::JS)) {
-        const FunctionNode *func = static_cast<const FunctionNode *>(node);
+    } else if (node->isFunction(Node::QML)) {
+        const auto *func = static_cast<const FunctionNode *>(node);
         if (!func->returnType().isEmpty())
             synopsis = typified(func->returnType(), true) + name;
         else
@@ -349,10 +274,12 @@ QString CppCodeMarker::markedUpQmlItem(const Node *node, bool summary)
     if (summary) {
         if (node->isPreliminary())
             extra += " (preliminary)";
-        else if (node->isDeprecated())
-            extra += " (deprecated)";
-        else if (node->isObsolete())
-            extra += " (obsolete)";
+        else if (node->isDeprecated()) {
+            if (const QString &version = node->deprecatedSince(); !version.isEmpty())
+                extra += " (deprecated since " + version + ")";
+            else
+                extra += " (deprecated)";
+        }
     }
 
     if (!extra.isEmpty()) {
@@ -370,22 +297,6 @@ QString CppCodeMarker::markedUpName(const Node *node)
     return name;
 }
 
-QString CppCodeMarker::markedUpFullName(const Node *node, const Node *relative)
-{
-    if (node->name().isEmpty())
-        return "global";
-
-    QString fullName;
-    for (;;) {
-        fullName.prepend(markedUpName(node));
-        if (node->parent() == relative || node->parent()->name().isEmpty())
-            break;
-        fullName.prepend("<@op>::</@op>");
-        node = node->parent();
-    }
-    return fullName;
-}
-
 QString CppCodeMarker::markedUpEnumValue(const QString &enumValue, const Node *relative)
 {
     if (!relative->isEnumType())
@@ -416,16 +327,6 @@ QString CppCodeMarker::markedUpIncludes(const QStringList &includes)
     return code;
 }
 
-QString CppCodeMarker::functionBeginRegExp(const QString &funcName)
-{
-    return QLatin1Char('^') + QRegExp::escape(funcName) + QLatin1Char('$');
-}
-
-QString CppCodeMarker::functionEndRegExp(const QString & /* funcName */)
-{
-    return "^\\}$";
-}
-
 /*
     @char
     @class
@@ -442,79 +343,69 @@ QString CppCodeMarker::functionEndRegExp(const QString & /* funcName */)
 QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
                                  const Location & /* location */)
 {
-    static QSet<QString> types;
-    static QSet<QString> keywords;
+    static QSet<QString> types{
+        QLatin1String("bool"),       QLatin1String("char"),    QLatin1String("double"),
+        QLatin1String("float"),      QLatin1String("int"),     QLatin1String("long"),
+        QLatin1String("short"),      QLatin1String("signed"),  QLatin1String("unsigned"),
+        QLatin1String("uint"),       QLatin1String("ulong"),   QLatin1String("ushort"),
+        QLatin1String("uchar"),      QLatin1String("void"),    QLatin1String("qlonglong"),
+        QLatin1String("qulonglong"), QLatin1String("qint"),    QLatin1String("qint8"),
+        QLatin1String("qint16"),     QLatin1String("qint32"),  QLatin1String("qint64"),
+        QLatin1String("quint"),      QLatin1String("quint8"),  QLatin1String("quint16"),
+        QLatin1String("quint32"),    QLatin1String("quint64"), QLatin1String("qreal"),
+        QLatin1String("cond")
+    };
 
-    if (types.isEmpty()) {
-        // initialize statics
-        Q_ASSERT(keywords.isEmpty());
-        static const QString typeTable[] = {
-            QLatin1String("bool"),       QLatin1String("char"),    QLatin1String("double"),
-            QLatin1String("float"),      QLatin1String("int"),     QLatin1String("long"),
-            QLatin1String("short"),      QLatin1String("signed"),  QLatin1String("unsigned"),
-            QLatin1String("uint"),       QLatin1String("ulong"),   QLatin1String("ushort"),
-            QLatin1String("uchar"),      QLatin1String("void"),    QLatin1String("qlonglong"),
-            QLatin1String("qulonglong"), QLatin1String("qint"),    QLatin1String("qint8"),
-            QLatin1String("qint16"),     QLatin1String("qint32"),  QLatin1String("qint64"),
-            QLatin1String("quint"),      QLatin1String("quint8"),  QLatin1String("quint16"),
-            QLatin1String("quint32"),    QLatin1String("quint64"), QLatin1String("qreal"),
-            QLatin1String("cond")
-        };
-
-        static const QString keywordTable[] = {
-            QLatin1String("and"), QLatin1String("and_eq"), QLatin1String("asm"),
-            QLatin1String("auto"), QLatin1String("bitand"), QLatin1String("bitor"),
-            QLatin1String("break"), QLatin1String("case"), QLatin1String("catch"),
-            QLatin1String("class"), QLatin1String("compl"), QLatin1String("const"),
-            QLatin1String("const_cast"), QLatin1String("continue"), QLatin1String("default"),
-            QLatin1String("delete"), QLatin1String("do"), QLatin1String("dynamic_cast"),
-            QLatin1String("else"), QLatin1String("enum"), QLatin1String("explicit"),
-            QLatin1String("export"), QLatin1String("extern"), QLatin1String("false"),
-            QLatin1String("for"), QLatin1String("friend"), QLatin1String("goto"),
-            QLatin1String("if"), QLatin1String("include"), QLatin1String("inline"),
-            QLatin1String("monitor"), QLatin1String("mutable"), QLatin1String("namespace"),
-            QLatin1String("new"), QLatin1String("not"), QLatin1String("not_eq"),
-            QLatin1String("operator"), QLatin1String("or"), QLatin1String("or_eq"),
-            QLatin1String("private"), QLatin1String("protected"), QLatin1String("public"),
-            QLatin1String("register"), QLatin1String("reinterpret_cast"), QLatin1String("return"),
-            QLatin1String("sizeof"), QLatin1String("static"), QLatin1String("static_cast"),
-            QLatin1String("struct"), QLatin1String("switch"), QLatin1String("template"),
-            QLatin1String("this"), QLatin1String("throw"), QLatin1String("true"),
-            QLatin1String("try"), QLatin1String("typedef"), QLatin1String("typeid"),
-            QLatin1String("typename"), QLatin1String("union"), QLatin1String("using"),
-            QLatin1String("virtual"), QLatin1String("volatile"), QLatin1String("wchar_t"),
-            QLatin1String("while"), QLatin1String("xor"), QLatin1String("xor_eq"),
-            QLatin1String("synchronized"),
-            // Qt specific
-            QLatin1String("signals"), QLatin1String("slots"), QLatin1String("emit")
-        };
-
-        types.reserve(sizeof(typeTable) / sizeof(QString));
-        for (int j = sizeof(typeTable) / sizeof(QString) - 1; j; --j)
-            types.insert(typeTable[j]);
-
-        keywords.reserve(sizeof(keywordTable) / sizeof(QString));
-        for (int j = sizeof(keywordTable) / sizeof(QString) - 1; j; --j)
-            keywords.insert(keywordTable[j]);
-    }
-#define readChar() ch = (i < code.length()) ? code[i++].cell() : EOF
+    static QSet<QString> keywords{
+        QLatin1String("and"), QLatin1String("and_eq"), QLatin1String("asm"), QLatin1String("auto"),
+        QLatin1String("bitand"), QLatin1String("bitor"), QLatin1String("break"),
+        QLatin1String("case"), QLatin1String("catch"), QLatin1String("class"),
+        QLatin1String("compl"), QLatin1String("const"), QLatin1String("const_cast"),
+        QLatin1String("continue"), QLatin1String("default"), QLatin1String("delete"),
+        QLatin1String("do"), QLatin1String("dynamic_cast"), QLatin1String("else"),
+        QLatin1String("enum"), QLatin1String("explicit"), QLatin1String("export"),
+        QLatin1String("extern"), QLatin1String("false"), QLatin1String("for"),
+        QLatin1String("friend"), QLatin1String("goto"), QLatin1String("if"),
+        QLatin1String("include"), QLatin1String("inline"), QLatin1String("monitor"),
+        QLatin1String("mutable"), QLatin1String("namespace"), QLatin1String("new"),
+        QLatin1String("not"), QLatin1String("not_eq"), QLatin1String("operator"),
+        QLatin1String("or"), QLatin1String("or_eq"), QLatin1String("private"),
+        QLatin1String("protected"), QLatin1String("public"), QLatin1String("register"),
+        QLatin1String("reinterpret_cast"), QLatin1String("return"), QLatin1String("sizeof"),
+        QLatin1String("static"), QLatin1String("static_cast"), QLatin1String("struct"),
+        QLatin1String("switch"), QLatin1String("template"), QLatin1String("this"),
+        QLatin1String("throw"), QLatin1String("true"), QLatin1String("try"),
+        QLatin1String("typedef"), QLatin1String("typeid"), QLatin1String("typename"),
+        QLatin1String("union"), QLatin1String("using"), QLatin1String("virtual"),
+        QLatin1String("volatile"), QLatin1String("wchar_t"), QLatin1String("while"),
+        QLatin1String("xor"), QLatin1String("xor_eq"), QLatin1String("synchronized"),
+        // Qt specific
+        QLatin1String("signals"), QLatin1String("slots"), QLatin1String("emit")
+    };
 
     QString code = in;
     QString out;
-    QStringRef text;
+    QStringView text;
     int braceDepth = 0;
     int parenDepth = 0;
     int i = 0;
     int start = 0;
     int finish = 0;
     QChar ch;
-    QRegExp classRegExp("Qt?(?:[A-Z3]+[a-z][A-Za-z]*|t)");
-    QRegExp functionRegExp("q([A-Z][a-z]+)+");
-    QRegExp findFunctionRegExp(QStringLiteral("^\\s*\\("));
+    static const QRegularExpression classRegExp(QRegularExpression::anchoredPattern("Qt?(?:[A-Z3]+[a-z][A-Za-z]*|t)"));
+    static const QRegularExpression functionRegExp(QRegularExpression::anchoredPattern("q([A-Z][a-z]+)+"));
+    static const QRegularExpression findFunctionRegExp(QStringLiteral("^\\s*\\("));
+    bool atEOF = false;
+
+    auto readChar = [&]() {
+         if (i < code.size())
+            ch = code[i++];
+         else
+            atEOF = true;
+    };
 
     readChar();
-
-    while (ch != QChar(EOF)) {
+    while (!atEOF) {
         QString tag;
         bool target = false;
 
@@ -524,11 +415,11 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
                 ident += ch;
                 finish = i;
                 readChar();
-            } while (ch.isLetterOrNumber() || ch == '_');
+            } while (!atEOF && (ch.isLetterOrNumber() || ch == '_'));
 
-            if (classRegExp.exactMatch(ident)) {
+            if (classRegExp.match(ident).hasMatch()) {
                 tag = QStringLiteral("type");
-            } else if (functionRegExp.exactMatch(ident)) {
+            } else if (functionRegExp.match(ident).hasMatch()) {
                 tag = QStringLiteral("func");
                 target = true;
             } else if (types.contains(ident)) {
@@ -544,7 +435,7 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
             do {
                 finish = i;
                 readChar();
-            } while (ch.isLetterOrNumber() || ch == '.');
+            } while (!atEOF && (ch.isLetterOrNumber() || ch == '.'));
             tag = QStringLiteral("number");
         } else {
             switch (ch.unicode()) {
@@ -573,7 +464,7 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
                 finish = i;
                 readChar();
 
-                while (ch != QChar(EOF) && ch != '"') {
+                while (!atEOF && ch != '"') {
                     if (ch == '\\')
                         readChar();
                     readChar();
@@ -585,7 +476,7 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
             case '#':
                 finish = i;
                 readChar();
-                while (ch != QChar(EOF) && ch != '\n') {
+                while (!atEOF && ch != '\n') {
                     if (ch == '\\')
                         readChar();
                     finish = i;
@@ -597,7 +488,7 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
                 finish = i;
                 readChar();
 
-                while (ch != QChar(EOF) && ch != '\'') {
+                while (!atEOF && ch != '\'') {
                     if (ch == '\\')
                         readChar();
                     readChar();
@@ -619,7 +510,7 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
             case ':':
                 finish = i;
                 readChar();
-                if (ch == ':') {
+                if (!atEOF && ch == ':') {
                     finish = i;
                     readChar();
                     tag = QStringLiteral("op");
@@ -628,11 +519,11 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
             case '/':
                 finish = i;
                 readChar();
-                if (ch == '/') {
+                if (!atEOF && ch == '/') {
                     do {
                         finish = i;
                         readChar();
-                    } while (ch != QChar(EOF) && ch != '\n');
+                    } while (!atEOF && ch != '\n');
                     tag = QStringLiteral("comment");
                 } else if (ch == '*') {
                     bool metAster = false;
@@ -642,9 +533,8 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
                     readChar();
 
                     while (!metAsterSlash) {
-                        if (ch == QChar(EOF))
+                        if (atEOF)
                             break;
-
                         if (ch == '*')
                             metAster = true;
                         else if (metAster && ch == '/')
@@ -675,7 +565,7 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
             }
         }
 
-        text = code.midRef(start, finish - start);
+        text = QStringView{code}.mid(start, finish - start);
         start = finish;
 
         if (!tag.isEmpty()) {
@@ -698,8 +588,8 @@ QString CppCodeMarker::addMarkUp(const QString &in, const Node * /* relative */,
         }
     }
 
-    if (start < code.length()) {
-        appendProtectedString(&out, code.midRef(start));
+    if (start < code.size()) {
+        appendProtectedString(&out, QStringView{code}.mid(start));
     }
 
     return out;
