@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the tools applications of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "cppcodeparser.h"
 
@@ -74,12 +49,9 @@ CppCodeParser::CppCodeParser()
                        << COMMAND_QMLPROPERTYGROUP // mws 13/03/2019
                        << COMMAND_QMLATTACHEDPROPERTY << COMMAND_QMLSIGNAL
                        << COMMAND_QMLATTACHEDSIGNAL << COMMAND_QMLMETHOD
-                       << COMMAND_QMLATTACHEDMETHOD << COMMAND_QMLBASICTYPE << COMMAND_QMLMODULE
-                       << COMMAND_JSTYPE << COMMAND_JSPROPERTY
-                       << COMMAND_JSPROPERTYGROUP // mws 13/03/2019
-                       << COMMAND_JSATTACHEDPROPERTY << COMMAND_JSSIGNAL << COMMAND_JSATTACHEDSIGNAL
-                       << COMMAND_JSMETHOD << COMMAND_JSATTACHEDMETHOD << COMMAND_JSBASICTYPE
-                       << COMMAND_JSMODULE << COMMAND_STRUCT << COMMAND_UNION;
+                       << COMMAND_QMLATTACHEDMETHOD << COMMAND_QMLVALUETYPE << COMMAND_QMLBASICTYPE
+                       << COMMAND_QMLMODULE
+                       << COMMAND_STRUCT << COMMAND_UNION;
     }
     if (metaCommands_.isEmpty()) {
         metaCommands_ = commonMetaCommands();
@@ -144,6 +116,8 @@ void CppCodeParser::initializeParser()
         m_exampleImageFilter = exampleImagePatterns.join(' ');
     else
         m_exampleImageFilter = "*.png";
+
+    m_showLinkErrors = !config.getBool(CONFIG_NOLINKERRORS);
 }
 
 /*!
@@ -285,13 +259,6 @@ Node *CppCodeParser::processTopicCommand(const Doc &doc, const QString &command,
         cn->setLocation(doc.startLocation());
         cn->markSeen();
         return cn;
-    } else if (command == COMMAND_JSMODULE) {
-        QStringList blankSplit = arg.first.split(QLatin1Char(' '));
-        CollectionNode *cn = m_qdb->addJsModule(blankSplit[0]);
-        cn->setLogicalModuleInfo(blankSplit);
-        cn->setLocation(doc.startLocation());
-        cn->markSeen();
-        return cn;
     } else if (command == COMMAND_PAGE) {
         Node::PageType ptype = Node::ArticlePage;
         QStringList args = arg.first.split(QLatin1Char(' '));
@@ -315,36 +282,19 @@ Node *CppCodeParser::processTopicCommand(const Doc &doc, const QString &command,
         auto *pn = new PageNode(m_qdb->primaryTreeRoot(), args[0], ptype);
         pn->setLocation(doc.startLocation());
         return pn;
-    } else if (command == COMMAND_QMLTYPE) {
+    } else if (command == COMMAND_QMLTYPE ||
+               command == COMMAND_QMLVALUETYPE ||
+               command == COMMAND_QMLBASICTYPE) {
         QmlTypeNode *qcn = nullptr;
+        auto nodeType = (command == COMMAND_QMLTYPE) ? Node::QmlType : Node::QmlValueType;
         Node *candidate = m_qdb->primaryTreeRoot()->findChildNode(arg.first, Node::QML);
-        if (candidate != nullptr && candidate->isQmlType())
-            qcn = static_cast<QmlTypeNode *>(candidate);
-        else
-            qcn = new QmlTypeNode(m_qdb->primaryTreeRoot(), arg.first);
+        qcn = (candidate && candidate->nodeType() == nodeType) ?
+                static_cast<QmlTypeNode *>(candidate) :
+                new QmlTypeNode(m_qdb->primaryTreeRoot(), arg.first, nodeType);
         qcn->setLocation(doc.startLocation());
         return qcn;
-    } else if (command == COMMAND_JSTYPE) {
-        QmlTypeNode *qcn = nullptr;
-        Node *candidate = m_qdb->primaryTreeRoot()->findChildNode(arg.first, Node::JS);
-        if (candidate != nullptr && candidate->isJsType())
-            qcn = static_cast<QmlTypeNode *>(candidate);
-        else
-            qcn = new QmlTypeNode(m_qdb->primaryTreeRoot(), arg.first, Node::JsType);
-        qcn->setLocation(doc.startLocation());
-        return qcn;
-    } else if (command == COMMAND_QMLBASICTYPE) {
-        auto *node = new QmlBasicTypeNode(m_qdb->primaryTreeRoot(), arg.first);
-        node->setLocation(doc.startLocation());
-        return node;
-    } else if (command == COMMAND_JSBASICTYPE) {
-        auto *node = new QmlBasicTypeNode(m_qdb->primaryTreeRoot(), arg.first, Node::JsBasicType);
-        node->setLocation(doc.startLocation());
-        return node;
     } else if ((command == COMMAND_QMLSIGNAL) || (command == COMMAND_QMLMETHOD)
-               || (command == COMMAND_QMLATTACHEDSIGNAL) || (command == COMMAND_QMLATTACHEDMETHOD)
-               || (command == COMMAND_JSSIGNAL) || (command == COMMAND_JSMETHOD)
-               || (command == COMMAND_JSATTACHEDSIGNAL) || (command == COMMAND_JSATTACHEDMETHOD)) {
+               || (command == COMMAND_QMLATTACHEDSIGNAL) || (command == COMMAND_QMLATTACHEDMETHOD)) {
         Q_UNREACHABLE();
     }
     return nullptr;
@@ -413,7 +363,6 @@ void CppCodeParser::processQmlProperties(const Doc &doc, NodeList &nodes, DocLis
     QString qmlTypeName;
 
     Topic topic = topics.at(0);
-    bool jsProps = isJSPropertyTopic(topic.m_topic);
     arg = topic.m_args;
     if (splitQmlPropertyArg(arg, type, module, qmlTypeName, property, doc.location())) {
         qsizetype i = property.indexOf('.');
@@ -423,14 +372,16 @@ void CppCodeParser::processQmlProperties(const Doc &doc, NodeList &nodes, DocLis
 
     NodeList sharedNodes;
     QmlTypeNode *qmlType = m_qdb->findQmlType(module, qmlTypeName);
+    // Note: Constructing a QmlType node by default, as opposed to QmlValueType.
+    // This may lead to unexpected behavior if documenting \qmlvaluetype's properties
+    // before the type itself.
     if (qmlType == nullptr)
-        qmlType = new QmlTypeNode(m_qdb->primaryTreeRoot(), qmlTypeName);
+        qmlType = new QmlTypeNode(m_qdb->primaryTreeRoot(), qmlTypeName, Node::QmlType);
 
     for (const auto &topicCommand : topics) {
         QString cmd = topicCommand.m_topic;
         arg = topicCommand.m_args;
-        if ((cmd == COMMAND_QMLPROPERTY) || (cmd == COMMAND_QMLATTACHEDPROPERTY)
-            || (cmd == COMMAND_JSPROPERTY) || (cmd == COMMAND_JSATTACHEDPROPERTY)) {
+        if ((cmd == COMMAND_QMLPROPERTY) || (cmd == COMMAND_QMLATTACHEDPROPERTY)) {
             bool attached = cmd.contains(QLatin1String("attached"));
             if (splitQmlPropertyArg(arg, type, module, qmlTypeName, property, doc.location())) {
                 if (qmlType != m_qdb->findQmlType(module, qmlTypeName)) {
@@ -446,20 +397,21 @@ void CppCodeParser::processQmlProperties(const Doc &doc, NodeList &nodes, DocLis
                     if (!doc.body().isEmpty()) {
                         doc.startLocation().warning(
                                 QStringLiteral("QML property documented multiple times: '%1'")
-                                        .arg(arg));
+                                        .arg(arg), QStringLiteral("also seen here: %1")
+                                                .arg(existingProperty->location().toString()));
                     }
                     continue;
                 }
                 auto *qpn = new QmlPropertyNode(qmlType, property, type, attached);
                 qpn->setLocation(doc.startLocation());
-                qpn->setGenus(jsProps ? Node::JS : Node::QML);
+                qpn->setGenus(Node::QML);
                 nodes.append(qpn);
                 docs.append(doc);
                 sharedNodes << qpn;
             }
         } else {
             doc.startLocation().warning(
-                    QStringLiteral("Command '\\%1'; not allowed with QML/JS property commands")
+                    QStringLiteral("Command '\\%1'; not allowed with QML property commands")
                             .arg(cmd));
         }
     }
@@ -468,8 +420,8 @@ void CppCodeParser::processQmlProperties(const Doc &doc, NodeList &nodes, DocLis
     // valid nodes. Note that it's important to do this *after* constructing
     // the topic nodes - which need to be written to index before the related
     // scn.
-    if (sharedNodes.count() > 1) {
-        auto *scn = new SharedCommentNode(qmlType, sharedNodes.count(), group);
+    if (sharedNodes.size() > 1) {
+        auto *scn = new SharedCommentNode(qmlType, sharedNodes.size(), group);
         scn->setLocation(doc.startLocation());
         nodes.append(scn);
         docs.append(doc);
@@ -586,20 +538,25 @@ void CppCodeParser::processMetaCommand(const Doc &doc, const QString &command,
     } else if (command == COMMAND_QMLINHERITS) {
         if (node->name() == arg)
             doc.location().warning(QStringLiteral("%1 tries to inherit itself").arg(arg));
-        else if (node->isQmlType() || node->isJsType()) {
+        else if (node->isQmlType()) {
             auto *qmlType = static_cast<QmlTypeNode *>(node);
             qmlType->setQmlBaseName(arg);
         }
     } else if (command == COMMAND_QMLINSTANTIATES) {
-        if (node->isQmlType() || node->isJsType()) {
+        if (node->isQmlType()) {
             ClassNode *classNode = m_qdb->findClassNode(arg.split("::"));
             if (classNode)
                 node->setClassNode(classNode);
-            else
+            else if (m_showLinkErrors) {
                 doc.location().warning(
-                        QStringLiteral("C++ class %1 not found: \\instantiates %1").arg(arg));
-        } else
-            doc.location().warning(QStringLiteral("\\instantiates is only allowed in \\qmltype"));
+                        QStringLiteral("C++ class %2 not found: \\%1 %2")
+                                .arg(command, arg));
+            }
+        } else {
+            doc.location().warning(
+                    QStringLiteral("\\%1 is only allowed in \\%2")
+                            .arg(command, COMMAND_QMLTYPE));
+        }
     } else if (command == COMMAND_DEFAULT) {
         if (!node->isQmlProperty()) {
             doc.location().warning(QStringLiteral("Ignored '\\%1', applies only to '\\%2'")
@@ -620,7 +577,7 @@ void CppCodeParser::processMetaCommand(const Doc &doc, const QString &command,
         else
             static_cast<QmlPropertyNode *>(node)->setRequired();
     } else if ((command == COMMAND_QMLABSTRACT) || (command == COMMAND_ABSTRACT)) {
-        if (node->isQmlType() || node->isJsType())
+        if (node->isQmlType())
             node->setAbstract(true);
     } else if (command == COMMAND_DEPRECATED) {
         node->setStatus(Node::Deprecated);
@@ -633,8 +590,6 @@ void CppCodeParser::processMetaCommand(const Doc &doc, const QString &command,
         m_qdb->addToModule(arg, node);
     } else if (command == COMMAND_INQMLMODULE) {
         m_qdb->addToQmlModule(arg, node);
-    } else if (command == COMMAND_INJSMODULE) {
-        m_qdb->addToJsModule(arg, node);
     } else if (command == COMMAND_OBSOLETE) {
         node->setStatus(Node::Deprecated);
     } else if (command == COMMAND_NONREENTRANT) {
@@ -697,7 +652,7 @@ void CppCodeParser::processMetaCommands(const Doc &doc, Node *node)
 }
 
 /*!
- Parse QML/JS signal/method topic commands.
+ Parse QML signal/method topic commands.
  */
 FunctionNode *CppCodeParser::parseOtherFuncArg(const QString &topic, const Location &location,
                                                const QString &funcArg)
@@ -713,7 +668,7 @@ FunctionNode *CppCodeParser::parseOtherFuncArg(const QString &topic, const Locat
     qsizetype firstBlank = funcName.indexOf(QChar(' '));
     if (firstBlank > 0) {
         returnType = funcName.left(firstBlank);
-        funcName = funcName.right(funcName.length() - firstBlank - 1);
+        funcName = funcName.right(funcName.size() - firstBlank - 1);
     }
 
     QStringList colonSplit(funcName.split("::"));
@@ -733,8 +688,6 @@ FunctionNode *CppCodeParser::parseOtherFuncArg(const QString &topic, const Locat
     funcName = colonSplit.last();
 
     Aggregate *aggregate = m_qdb->findQmlType(moduleName, elementName);
-    if (aggregate == nullptr)
-        aggregate = m_qdb->findQmlBasicType(moduleName, elementName);
     if (aggregate == nullptr)
         return nullptr;
 
@@ -787,7 +740,7 @@ FunctionNode *CppCodeParser::parseMacroArg(const Location &location, const QStri
             params = afterParen.left(rightParen);
     }
     int i = 0;
-    while (i < macroName.length() && !macroName.at(i).isLetter())
+    while (i < macroName.size() && !macroName.at(i).isLetter())
         i++;
     if (i > 0) {
         returnType += QChar(' ') + macroName.left(i);
@@ -802,8 +755,9 @@ FunctionNode *CppCodeParser::parseMacroArg(const Location &location, const QStri
     macro->setReturnType(returnType);
     macro->setParameters(params);
     if (macro->compare(oldMacroNode)) {
-        location.warning(QStringLiteral("\\macro %1 documented more than once").arg(macroArg));
-        oldMacroNode->doc().location().warning(QStringLiteral("(The previous doc is here)"));
+        location.warning(QStringLiteral("\\macro %1 documented more than once")
+                .arg(macroArg), QStringLiteral("also seen here: %1")
+                        .arg(oldMacroNode->doc().location().toString()));
     }
     return macro;
 }
@@ -866,16 +820,6 @@ void CppCodeParser::setExampleFileLists(ExampleNode *en)
     en->setImages(imageFiles);
 }
 
-/*!
-  returns true if \a t is \e {jssignal}, \e {jsmethod},
-  \e {jsattachedsignal}, or \e {jsattachedmethod}.
- */
-bool CppCodeParser::isJSMethodTopic(const QString &t)
-{
-    return (t == COMMAND_JSSIGNAL || t == COMMAND_JSMETHOD || t == COMMAND_JSATTACHEDSIGNAL
-            || t == COMMAND_JSATTACHEDMETHOD);
-}
-
 /*!
   returns true if \a t is \e {qmlsignal}, \e {qmlmethod},
   \e {qmlattachedsignal}, or \e {qmlattachedmethod}.
@@ -886,15 +830,6 @@ bool CppCodeParser::isQMLMethodTopic(const QString &t)
             || t == COMMAND_QMLATTACHEDMETHOD);
 }
 
-/*!
-  Returns true if \a t is \e {jsproperty}, \e {jspropertygroup},
-  or \e {jsattachedproperty}.
- */
-bool CppCodeParser::isJSPropertyTopic(const QString &t)
-{
-    return (t == COMMAND_JSPROPERTY || t == COMMAND_JSATTACHEDPROPERTY);
-}
-
 /*!
   Returns true if \a t is \e {qmlproperty}, \e {qmlpropertygroup},
   or \e {qmlattachedproperty}.
@@ -907,7 +842,7 @@ bool CppCodeParser::isQMLPropertyTopic(const QString &t)
 void CppCodeParser::processTopicArgs(const Doc &doc, const QString &topic, NodeList &nodes,
                                      DocList &docs)
 {
-    if (isQMLPropertyTopic(topic) || isJSPropertyTopic(topic)) {
+    if (isQMLPropertyTopic(topic)) {
         processQmlProperties(doc, nodes, docs);
     } else {
         ArgList args = doc.metaCommandArgs(topic);
@@ -918,7 +853,7 @@ void CppCodeParser::processTopicArgs(const Doc &doc, const QString &topic, NodeL
                     node = parserForLanguage("Clang")->parseFnArg(doc.location(), args[0].first, args[0].second);
             } else if (topic == COMMAND_MACRO) {
                 node = parseMacroArg(doc.location(), args[0].first);
-            } else if (isQMLMethodTopic(topic) || isJSMethodTopic(topic)) {
+            } else if (isQMLMethodTopic(topic)) {
                 node = parseOtherFuncArg(topic, doc.location(), args[0].first);
             } else if (topic == COMMAND_DONTDOCUMENT) {
                 m_qdb->primaryTree()->addToDontDocumentMap(args[0].first);
@@ -931,14 +866,14 @@ void CppCodeParser::processTopicArgs(const Doc &doc, const QString &topic, NodeL
             }
         } else if (args.size() > 1) {
             QList<SharedCommentNode *> sharedCommentNodes;
-            for (const auto &arg : qAsConst(args)) {
+            for (const auto &arg : std::as_const(args)) {
                 node = nullptr;
                 if (topic == COMMAND_FN) {
                     if (showInternal() || !doc.isInternal())
                         node = parserForLanguage("Clang")->parseFnArg(doc.location(), arg.first, arg.second);
                 } else if (topic == COMMAND_MACRO) {
                     node = parseMacroArg(doc.location(), arg.first);
-                } else if (isQMLMethodTopic(topic) || isJSMethodTopic(topic)) {
+                } else if (isQMLMethodTopic(topic)) {
                     node = parseOtherFuncArg(topic, doc.location(), arg.first);
                 } else {
                     node = processTopicCommand(doc, topic, arg);
@@ -995,10 +930,10 @@ void CppCodeParser::processMetaCommands(NodeList &nodes, DocList &docs)
 bool CppCodeParser::hasTooManyTopics(const Doc &doc) const
 {
     const QSet<QString> topicCommandsUsed = topicCommands() & doc.metaCommandsUsed();
-    if (topicCommandsUsed.count() > 1) {
+    if (topicCommandsUsed.size() > 1) {
         bool ok = true;
         for (const auto &t : topicCommandsUsed) {
-            if (!t.startsWith(QLatin1String("qml")) && !t.startsWith(QLatin1String("js")))
+            if (!t.startsWith(QLatin1String("qml")))
                 ok = false;
         }
         if (ok)
